@@ -2,7 +2,7 @@ package Redis::Fast;
 
 BEGIN {
     use XSLoader;
-    our $VERSION = '0.14';
+    our $VERSION = '0.15';
     XSLoader::load __PACKAGE__, $VERSION;
 }
 
@@ -387,7 +387,7 @@ sub __process_pubsub_msg {
   my $sub   = $m->[1];
   my $cbid  = "$m->[0]:$sub";
   my $data  = pop @$m;
-  my $topic = $m->[2] || $sub;
+  my $topic = defined $m->[2] ? $m->[2] : $sub;
 
   if (!exists $subs->{$cbid}) {
     warn "Message for topic '$topic' ($cbid) without expected callback, ";
